@@ -1,44 +1,42 @@
+from __future__ import annotations
+import typing
 import weakref
 import uuid
 import traceback
 import time
 
+from pyhtmlgui import Observable
+from pyhtmlgui.pyhtmlguiInstance import PyHtmlGuiInstance
 
-class PyHtmlView():
+
+class PyHtmlView:
     TEMPLATE_FILE = None
     TEMPLATE_STR = None
     WRAPPER_ELEMENT = "pyHtmlView"
     WRAPPER_EXTRAS = ""
 
-    def __init__(self, subject, parent):
+    def __init__(self,
+                 subject: object,
+                 parent:  typing.Union[PyHtmlView, PyHtmlGuiInstance],
+                 **kwargs):
 
-        # Component state and data
         self.uid = "%s" % uuid.uuid4()
         self.is_visible = False
-        self._children = weakref.WeakSet()
 
-        # Weak references to  subject, parent
-        if type(parent) == weakref.ref:
-            parent = parent()
-        if type(subject) == weakref.ref:
-            subject = subject()
+        parent._add_child(self)
+        self._subject_wref = weakref.ref(subject, self._on_subject_died)
+        self._parent_wref = weakref.ref(parent, None)
 
-        self._subject_wref = None
-        self._parent_wref = None
         self._subject = None  # this is replace for a short time on render by the actual resolved object
-        if subject is not None:
-            self._subject_wref = weakref.ref(subject, self._on_subject_died)  # non gui object we reprecent or talk to
-        if parent is not None:
-            parent._add_child(self)
-            self._parent_wref = weakref.ref(parent, None)  # parent of this element
-
-        # get template loader function from parent
-        self._get_template = parent._get_template
-        self._was_rendered = True
-        self._last_rendered = None  # timestamp of last rendering, for debug only
-
-        self._call_javascript = parent._call_javascript  # root element gets this supplied by pyhtmlgui lib, else none
+        self._children = weakref.WeakSet()
         self._observables = []
+        self.__last_rendered = None  # timestamp of last rendering, for debug only
+        self.__was_rendered = False
+
+        if type(parent) == PyHtmlGuiInstance:
+            self._instance = parent
+        else:
+            self._instance = parent._instance
 
         # attach default observation event and detach because default componens is invisible until rendered
         if self._on_subject_updated is not None:
@@ -55,31 +53,41 @@ class PyHtmlView():
         return self._subject_wref()
 
     @property
-    def parent(self):
+    def parent(self) -> PyHtmlView:
         return self._parent_wref()
 
-    def _on_subject_updated(self, source, **kwargs):
+    def _on_subject_updated(self, source, **kwargs) -> None:
         self.update()
 
-    def _on_subject_died(self, wr):
+    # noinspection PyUnusedLocal
+    def _on_subject_died(self, wr) -> None:
         self.delete()
 
-    # return html string rendered from template
-    # automatically set component to visible
-    def render(self):
+    def render(self) -> typing.Union[str, None]:
+        """
+            Return object rendered to html string. This function should be called from inside the jinja templates.
+            Direct usage is not needed, Return html string rendered from template
+        """
         html = self._inner_html()
         if html is None:
             return None
-        self._last_rendered = time.time()
+        self.__last_rendered = time.time()
         if self.WRAPPER_ELEMENT is None:
             return html
         else:
-            return "<%s id='%s' %s data-pyhtmlgui-class='%s'>%s</%s>" % (
-                self.WRAPPER_ELEMENT, self.uid, self.WRAPPER_EXTRAS, self.__class__.__name__, html,
-                self.WRAPPER_ELEMENT)
+            s = "<%(w_el)s id='%(uid)s' %(w_ex)s data-pyhtmlgui-class='(data_class)'>%(content)s</%(w_el)s>"
+            return s % {
+                "w_el" : self.WRAPPER_ELEMENT,
+                "w_ex" : self.WRAPPER_EXTRAS,
+                "uid" : self.uid,
+                "data_class" : self.__class__.__name__,
+                "content" : html,
+            }
 
-    # update rendered component in place, must be visible
-    def update(self):
+    def update(self) -> None:
+        """
+        update rendered view in DOM, must be visible
+        """
         if self.is_visible is True:
             html_content = self.render()
             if html_content is not None:  # object might have died, in that case don't render
@@ -87,25 +95,36 @@ class PyHtmlView():
         else:
             raise Exception("Can't update invisible components")
 
-    def insert_element(self, index, element):
+    def insert_element(self, index: int, element: PyHtmlView) -> bool:
+        """
+        Insert a new element into DOM at index.
+        This is used for example in ObservableDictView and ObservableListView to insert newly created elements.
+        """
         html_content = element.render()
         if html_content is not None:  # object might have died, in that case don't render
             self.call_javascript("pyhtmlgui.insert_element", [self.uid, index, html_content], skip_results=True)
             return True
         return False
 
-    # detach events, remove from parent, remove from frontend if is visible
-    def delete(self, already_removed_from_dom=False):
-        if self.is_visible is True and already_removed_from_dom is False:
+    def delete(self, remove_from_dom: bool = True) -> None:
+        """
+        Delete element, detach events, remove from parent, remove from frontend if element is visible
+
+        :param remove_from_dom: If parent element calls delete, remove_from_dom is set to False,
+                                because parent removes itself + all childen from dom automatically
+        """
+        if self.is_visible is True and remove_from_dom is True:
             self.call_javascript("pyhtmlgui.remove_element", [self.uid], skip_results=True)
         self.set_visible(False)
         self.parent._remove_child(self)
         for child in self._children:
-            child.delete(already_removed_from_dom=True)
+            child.delete(remove_from_dom=False)
+
+    def set_visible(self, visible: bool) -> None:
+        """
+        Set component and childens visibility, components that are not visible get their events detached
+        """
 
-    # set component and childens visibility
-    # components that are not visible get their events detached
-    def set_visible(self, visible):
         if self.is_visible == visible:
             return
 
@@ -114,7 +133,7 @@ class PyHtmlView():
             for observable, target in self._observables:
                 try:
                     observable().detach_observer(target())  # resolve weak references
-                except:
+                except Exception:
                     pass
             for child in self._children:
                 child.set_visible(False)
@@ -123,39 +142,49 @@ class PyHtmlView():
             for observable, target in self._observables:
                 try:
                     observable().attach_observer(target())  # resolve weak references
-                except:
+                except Exception:
                     pass
 
-    # function so we have function arguments name completion in editor,
-    # in theorie we could directly use self._call_javascript
     def call_javascript(self, js_function_name, args, skip_results=False):
-        return self._call_javascript(js_function_name, args, skip_results)
+        """
+        Call javascript function in frontend.
+        :param js_function_name: Name of javascript function
+        :param args: Arguments for js function
+        :param skip_results: Don't receive results, give some slight performance inprovement because
+                             we don't wait for results
+        """
+        return self._instance.call_javascript(js_function_name, args, skip_results)
 
-    # this is a convinience function, you could also call the subcall directly
     def eval_javascript(self, script, skip_results=False, **kwargs):
+        """
+        Run script in frontend.
+        :param script: Javascript source code
+        :param skip_results: Don't receive results, give some slight performance inprovement because
+                             we don't wait for results
+        :param kwargs: kwargs are passed to javascript as "arg" variable
+        :return:
+        """
         if self.is_visible is False:
             raise Exception("Can't javascript_call invisible components")
-        return self.call_javascript("pyhtmlgui.eval_script", [script, kwargs], skip_results=skip_results)
+        return self._instance.call_javascript("pyhtmlgui.eval_script", [script, kwargs], skip_results=skip_results)
 
     def eval_javascript_electron(self, script, skip_results=False, **kwargs):
         if self.is_visible is False:
             raise Exception("Can't javascript_call invisible components")
-        return self.call_javascript("electron.eval_script", [script, kwargs], skip_results=skip_results)
+        return self._instance.call_javascript("electron.eval_script", [script, kwargs], skip_results=skip_results)
 
-    def _inner_html(self):
+    def _inner_html(self) -> typing.Union[str, None]:
         self._subject = self.subject  # receive hard reference to obj to it does not die on us while rendering
-        if self._subject is None:  # Observed oject died before render
+        if self._subject is None:  # Observed object died before render
             return None
-        # It should be ok to set visible here, althou this is befor the rendered object actually appeart in the dom.
-        # However, it should arrive at to dom before any other things that might be triggered because the object is visible,
-        # because of the websocket event loop
+
         self.set_visible(True)
         for child in self._children:
-            child._was_rendered = False
+            child.__was_rendered = False
 
         try:
-            html = self._get_template(self).render({"pyview": self})
-        except Exception as e:
+            html = self._instance.get_template(self).render({"pyview": self})
+        except Exception:
             tb = traceback.format_exc()
             msg = " Exception while rendering Template: %s\n" % self.__class__.__name__
             msg += " %s" % tb.replace("\n", "\n  ").strip()
@@ -164,24 +193,23 @@ class PyHtmlView():
             print(msg)
 
         # set children that have not been rendered in last pass to invisible
-        [c.set_visible(False) for c in self._children if c._was_rendered is False and c.is_visible is True]
+        [c.set_visible(False) for c in self._children if c.__was_rendered is False and c.is_visible is True]
 
-        self._was_rendered = True
+        self.__was_rendered = True
         self._subject = None  # remove hard reference to subject, it may die now
         return html
 
-    def _add_child(self, child):
+    def _add_child(self, child: PyHtmlView) -> None:
         self._children.add(child)
 
-    def _remove_child(self, child):
+    def _remove_child(self, child: PyHtmlView) -> None:
         self._children.remove(child)
 
-    # default target_function is  self._on_subject_updated
-    def add_observable(self, subject, target_function=None):
+    def add_observable(self, subject: Observable, target_function: typing.Callable = None) -> None:
         try:
             if not callable(subject.attach_observer) or not callable(subject.detach_observer):
                 raise Exception("object type '%s' can not be observed" % type(subject))
-        except:
+        except Exception:
             raise Exception("object type '%s' can not be observed" % type(subject))
         if target_function is None:
             target_function = self._on_subject_updated
@@ -192,7 +220,7 @@ class PyHtmlView():
             target_function = weakref.WeakMethod(target_function)
         self._observables.append([subject, target_function])
 
-    def remove_observable(self, subject, target_function=None):
+    def remove_observable(self, subject: Observable, target_function: typing.Callable = None) -> None:
         if target_function is None:
             target_function = self._on_subject_updated
         to_remove = []
